#include <12F629.H>
#fuses HS,WDT
#use delay(clock=20000000,restart_wdt)
#use rs232(baud=9600, xmit=PIN_A1, rcv=PIN_A3)

int8  DS_Presence;
int8  DS_SC;
int8  DS_Error;
int8  DS_Data;
int8  d_pin;

//function prototypes
int8 Read_byte(int8 sensor_nmbr);
void DS_Reset(int8 sensor_nmbr);
void Write_byte(int8 Data, int8 sensor_nmbr);
void read_convert(void);
void start_convert(void);

//********************************************************************
void main()
{
	int8 byte_read;

	//setup_adc_ports(NO_ANALOGS);//disable A/D converter
	//setup_adc(ADC_OFF);
	*25=7;//disable comparator

	delay_ms(200);
	start_convert();//start conversions on both sensors
	delay_ms(1000);//allow time for first conversion

	while(1)//continuous loop
	{
		while( !kbhit() )//while nothing received
			RESTART_WDT();//pet the dog

		byte_read = getc();//read one byte from host via USB

		if(byte_read == 'R')//if byte_read is 'R' then read
			read_convert();	//both devices and start new conversion

		if(byte_read == 'P')//if byte_read is 'P' then send response 'Q'
			putc('Q');

		byte_read=0;
	}
}//end of main()

//*********************************************************************
void read_convert()
{
   int8 tt, data;

	//read and return 9 bytes from sensor 1
	DS_Reset(1);//issue reset and wait for response
	if(DS_Error == 99)//sensor ready!
	{
		Write_byte(0xCC, 1);//skip ROM
		Write_byte(0xBE, 1);//request data read
		delay_ms(3);//short pause
		for(tt=0; tt<9; tt++)
		{
			putc( Read_byte(1) );//read 9 bytes and return to host
			delay_ms(3);
		}
	}
    else//no sensor present
	{
		for(tt=0; tt<9; tt++)
		{
			putc(0);//send all zeros
			delay_ms(3);
		}
	}

	//read and return 9 bytes from sensor 2
	DS_Reset(2);//issue reset and wait for response
	if(DS_Error == 99)//sensor ready!
	{
		Write_byte(0xCC, 2);//skip ROM
		Write_byte(0xBE, 2);//request data read
		delay_ms(30);//short pause
		for(tt=3; tt<12; tt++)//read 9 bytes and return to host
		{
			data = Read_byte(2); 
			putc( data );
			delay_ms(3);
		}
	}
    else//no sensor present
	{
		for(tt=0; tt<9; tt++)//sends all zeros
		{
			putc(0);//send all zeros
			delay_ms(3);
		}
	}

	start_convert();//start conversions on both temperature sensors
}

//*********************************************************************
void start_convert()//start conversions on both temperature sensors
{
	//start temperature conversion on sensor 1
	DS_Reset(1);//issue reset and wait for response
	if(DS_Error == 99)//no error - sensor ready!
	{	//if ready, start conversion
		Write_byte(0xcc, 1);//skip ROM
		Write_byte(0x44, 1);//start convert
	}

	//start temperature conversion on sensor 2
	DS_Reset(2);//issue reset and wait for response
	if(DS_Error == 99)//no error - sensor ready!
	{	//if ready, start conversion
		Write_byte(0xcc, 2);//skip ROM
		Write_byte(0x44, 2);//start convert
	}
}

//**********************************************************************
void DS_Reset(int8 sensor_nmbr)
{
	int8 count;

	DS_Error=99; //Reset the status bit to no error

	if(sensor_nmbr == 1)//release port pin
		input(PIN_A0);
	else
		input(PIN_A2);
	
	if(sensor_nmbr == 1)//Send start
		output_low(PIN_A0);
	else
		output_low(PIN_A2);

	delay_us(480);
	
	if(sensor_nmbr == 1)//release for reading
		input(PIN_A0);
	else
		input(PIN_A2);

	
	delay_us(90);//delay to about the center of the timeslot

	if(sensor_nmbr == 1)
		DS_Presence = input(PIN_A0);  //get presence status 1=fail 0=present
	else
		DS_Presence = input(PIN_A2);

	delay_us(300);

	if(sensor_nmbr == 1)
		DS_SC = input(PIN_A0);//check for short circuit
	else
		DS_SC = input(PIN_A2);

	if(DS_SC == 0)  //if short circuit then error
		DS_Error = 8;

	if(DS_Presence == 1)   //if no device detected then error
		DS_Error = 2;

	delay_us(10);
}


//**********************************************************************
int8 Read_byte(int8 sensor_nmbr)
{
	int8 data, count;

	data=0;
	for(count=8; count>0; count--)
	{
		if(sensor_nmbr == 1)//send start
			output_low(PIN_A0);
		else
			output_low(PIN_A2);

		delay_us(2);
		
		if(sensor_nmbr == 1)//Release for reading
			input(PIN_A0);
		else
			input(PIN_A2);

		delay_us(10);//delay to center of timeslot

		if(sensor_nmbr == 1)
			DS_Data=input(PIN_A0); //Read the data bit in
		else
			DS_Data=input(PIN_A2);

		delay_us(60);//Finish the timeslot

		//Put data into the byte
		data = data >> 1;  //Shift previous bits across
		if(DS_Data==1)      //If data was high then add high bit to data
			data +=0x80;

		delay_us(10);//Delay Between Bits
	}
	
	delay_us(10);//rise time plus min space

	return(data);
}

//**********************************************************************
void Write_byte(int8 Data, int8 sensor_nmbr)
{
	int8 count;

	for(count=8; count>0; count--)
	{
		DS_Data= Data & 0x01; //LSB

		if(sensor_nmbr == 1)//write the bit to the port
			output_low(PIN_A0);
		else
			output_low(PIN_A2);

		delay_us(4); //time slot start
		
		if(DS_Data)//write data to the port
		{	//float high
			if(sensor_nmbr == 1)
				input(PIN_A0);
			else
				input(PIN_A2);
		}
		else//drive low
		{
			if(sensor_nmbr == 1)
				output_low(PIN_A0);
			else
				output_low(PIN_A2);
		}

		delay_us(66);

		if(sensor_nmbr == 1)//release port pin
			input(PIN_A0);
		else
			input(PIN_A2);

		delay_us(10);

		Data>>=1; //shift next bit into place
	}

	delay_us(10);
}







